module Tpt
  module Rails
  end
end

require 'tpt/rails/engine'
require 'tpt/rails/internal'
require 'tpt/rails/config'
require 'tpt/rails/puma_stats_collector'

module Tpt::Rails
  class NotConfigured < StandardError; end

  class << self
    # USAGE:
    #
    # In your app in `config/initializers/tpt_rails.rb`:
    #
    #   Tpt::Rails.configure do |config|
    #     config.app_name = '…'
    #     config.app_env = '…'
    #     config.rollbar_access_token = '…'
    #     config.rollbar_enabled = true/false
    #     config.datadog_statsd_url = '…'
    #     config.health_check(:foo) { … }
    #     …
    #   end
    #
    # See Tpt::Rails::Config in `lib/tpt/rails/config.rb` for a list of all configs.
    def configure
      @config = Tpt::Rails::Config.new

      yield @config if block_given?

      @configured = true
      @config.setup
    end

    # Whether the `Tpt::Rails.configure` block has been run yet by the host application.
    def configured?
      @configured
    end

    # Returns the app name of the application. E.g. 'earnings'.
    delegate :app_name, to: :config

    # Returns the app environment of the application. E.g. 'production'/'integration'/'jordan123'.
    delegate :app_env, to: :config

    # Access to the `Datadog::Statsd` instance for this app.
    #
    # Examples:
    #
    #   Tpt::Rails.statsd.increment('foo', tags: ['a:b', 'c:d'])
    #   Tpt::Rails.statsd.event('Some title', 'Some text', alert_type: 'warning', tags: ['a:b'])
    #
    delegate :statsd, to: :config

    # Report an exception via an exception-reporting tool (e.g. Busgnag/Rollbar)
    #
    # Example:
    #
    #   rescue => e
    #     Tpt::Rails.report(e)
    #
    delegate :report, to: :config

    def config # :nodoc:
      configured? ? @config : raise(NotConfigured)
    end

    # This should be set by the deployment tool. E.g. Jenkins: https://git.io/JJFiD
    def release_version
      ENV['TPT_RELEASE_VERSION'].presence
    end
  end
end
