#
# Configures Datadog metrics and tracing, based on Tpt::Rails.config
#

require 'datadog/statsd'
require 'ddtrace'

class Tpt::Rails::Internal::Datadog # :nodoc:
  DEFAULT_STATSD_PORT = 8125
  DEFAULT_TRACE_PORT = 8126

  attr_reader :statsd

  def initialize(statsd_url: nil, trace_url: nil)
    @statsd_url = statsd_url
    @trace_url = trace_url

    @statsd = create_statsd

    if @trace_url
      configure_tracing
    end
  end

  private

  def create_statsd
    # Fall back to a dummy client so callers don't have to worry about null checking the client
    url = @statsd_url || 'statsd://localhost'
    uri = URI.parse(url)

    tags = ["env:#{Tpt::Rails.app_env}"]
    tags << ["tpt_release_version:#{Tpt::Rails.release_version}"] if Tpt::Rails.release_version

    ::Datadog::Statsd.new(
      uri.host,
      uri.port || DEFAULT_STATSD_PORT,
      namespace: Tpt::Rails.app_name,
      tags: tags,
    )
  end

  def configure_tracing
    uri = URI.parse(@trace_url)
    tags = { 'env' => Tpt::Rails.app_env }
    tags["tpt_release_version"] = Tpt::Rails.release_version if Tpt::Rails.release_version

    ::Datadog.configure do |c|
      # This will activate auto-instrumentation for Rails
      c.use :rails, {
        analytics_enabled: true,
        service_name: Tpt::Rails.app_name,
      }

      c.tracer(
        enabled: true,
        hostname: uri.host,
        port: uri.port || DEFAULT_TRACE_PORT,
        tags: tags,
      )

      c.logger.level = ::Logger::WARN
    end

    ::Datadog::Pipeline.before_flush(
      ::Datadog::Pipeline::SpanFilter.new { |span|
        span.resource =~ /TptRails::HealthChecksController/
      },
    )
  end
end
