class Tpt::Rails::Internal::HealthChecks # :nodoc:
  def initialize
    @checks = {}
  end

  def add(name, &block)
    @checks[name] = block
  end

  def check
    @checks.transform_values(&:call)
  end

  def setup
    setup_active_record
    setup_redis
  end

  private

  def setup_active_record
    if defined?(ActiveRecord::Base) && ActiveRecord::Base.configurations.any?
      add(:database) { ::ApplicationRecord.connection.select_value('select 1') == 1 }
    end
  end

  def setup_redis
    if Tpt::Rails.config.redis_url.present?
      add(:redis) { Redis.current.ping == 'PONG' }
    end
  end
end
