module Tpt::Rails::PumaStatsCollector # :nodoc:
  class PumaStats
    def initialize(stats)
      @stats = JSON.parse(stats, symbolize_names: true)
    end

    def clustered?
      @stats.has_key?(:workers)
    end

    def workers
      @stats.fetch(:workers, 1)
    end

    def booted_workers
      @stats.fetch(:booted_workers, 1)
    end

    def running
      get_stat(:running)
    end

    def backlog
      get_stat(:backlog)
    end

    def pool_capacity
      get_stat(:pool_capacity)
    end

    def max_threads
      get_stat(:max_threads)
    end

    def to_s
      {
        workers: workers,
        booted_workers: booted_workers,
        running: running,
        backlog: backlog,
        pool_capacity: pool_capacity,
        max_threads: max_threads
      }.to_json
    end

    private
    def get_stat(name)
      if clustered?
        @stats[:worker_status].map { |s| s[:last_status].fetch(name, 0) }.sum
      else
        @stats.fetch(name, 0)
      end
    end
  end

  class << self
    def run(metrics_client:, start_delay: 10, collect_interval: 30)
      Thread.new do
        # Give Puma time to start
        sleep start_delay

        # Get pod name if set
        pod_name = ENV["KUBE_POD"]

        loop do
          begin
            stats = PumaStats.new(Puma.stats)

            tags = []
            # Add pod_name tag if set
            tags << "pod_name:#{pod_name}" if pod_name

            metrics_client.gauge("puma.workers", stats.workers, tags: tags)
            metrics_client.gauge("puma.booted_workers", stats.booted_workers, tags: tags)
            metrics_client.gauge("puma.running", stats.running, tags: tags)
            metrics_client.gauge("puma.backlog", stats.backlog, tags: tags)
            metrics_client.gauge("puma.pool_capacity", stats.pool_capacity, tags: tags)
            metrics_client.gauge("puma.max_threads", stats.max_threads, tags: tags)
          end

          sleep collect_interval
        end
      end
    end
  end
end
